#ifdef WIN32
  #include <windows.h>
#endif

#include <stdlib.h> 
#include <GL/glut.h>
#include "primitives.h" 
#include "image.h"
 
 

tpImage * bobfront1, * bobfront2, * bobback, 
        * bobleft,   * bobright, 
        * bobtop,    * bobbottom,      /* Texturas da personagem */ 
        * earth; 
GLuint    texHandle[6], tempTexHandle; /* Identificadores das texturas armazenadas */ 
void InitLighting(); 

void Init(void) 
{ 
   /* Seleciona a cor negra como cor de fundo                 */
   glClearColor (0.0, 0.0, 0.0, 0.0); 
   /* Habilita o teste de profundidade (Zbuffer)              */ 
   /* Um pixel  desenhado com a cor da superfcie se esta tem*/ 
   /* profundidade menor ou igual ao valor atual armazenado no*/ 
   /* map de profundidades                                    */ 
   glEnable(GL_DEPTH_TEST); 
   /* Faz com que as normais transformadas sejam normalizadas */ 
   glEnable(GL_NORMALIZE); 
 
   /* glHint  uma funo que estabelece o comportamento      */ 
   /* da OpenGL em relao a aspectos especificados pelo      */ 
   /* primeiro parmetro que  uma constante.                 */ 
   /* No caso, os parmetros passados para a funo indicam   */ 
   /* que a OpenGL deve usar correo de perspectiva na       */ 
   /* interpolao das cores e/ou coordenadas de textura dos  */ 
   /* fragmentos(pixels). GL_NICEST indica que o OpenGL deve  */ 
   /* interpolao com correo de perspectiva e no utilizar */ 
   /* aproximaes                                            */ 
   glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);   
 
   /* Configura a iluminao                                  */ 
   InitLighting(); 
   /* Cria listas de display para cubos e quadricas           */ 
   InitCubesDisplayLists(); 
   InitQuadrics(); 
 
   /* Le as imagens das texturas                              */ 
   bobfront1    = ReadPpmImage2RGBA("bobfront1.ppm"); 
   bobfront2    = ReadPpmImage2RGBA("bobfront2.ppm"); 
   bobback      = ReadPpmImage2RGBA("bobback.ppm"); 
   bobleft      = ReadPpmImage2RGBA("bobleft.ppm"); 
   bobright     = ReadPpmImage2RGBA("bobright.ppm"); 
   bobtop       = ReadPpmImage2RGBA("bobtop.ppm"); 
   bobbottom    = ReadPpmImage2RGBA("bobbottom.ppm"); 
   earth        = ReadPpmImage2RGBA("earthhigh.ppm"); 
 
   /* Cria texturas armazenadas na placa a partir das imagens fornecidas   */ 
   /* O parmetro GL_LINEAR indica que a amplicao e reduo de polgonos */ 
   /* texturizados deve utilizar uma amostragem com filtragem linear       */ 
 
   texHandle[0] = CreateTexture(bobback,    GL_LINEAR, GL_LINEAR); 
   texHandle[1] = CreateTexture(bobright,   GL_LINEAR, GL_LINEAR); 
   texHandle[2] = CreateTexture(bobtop,     GL_LINEAR, GL_LINEAR); 
   texHandle[3] = CreateTexture(bobleft,    GL_LINEAR, GL_LINEAR); 
   texHandle[4] = CreateTexture(bobbottom,  GL_LINEAR, GL_LINEAR); 
   texHandle[5] = CreateTexture(bobfront1,  GL_LINEAR, GL_LINEAR); 
   texHandle[6] = CreateTexture(bobfront2,  GL_LINEAR, GL_LINEAR); 
   CreateTexture(earth, GL_LINEAR, GL_LINEAR); 
 
} 
 
 
 
void InitLighting() 
{ 
 GLfloat mat_specular[]         = {1.0,1.0,1.0,1.0}; 
 GLfloat mat_shininess[]        = {100.0}; 
 GLfloat light_position1[]      = {200.0,200.0,200.0,1.0}; 
 GLfloat light_position2[]      = {-500.0,500.0,0.0,1.0}; 
/* GLfloat white_light_specular[] = {0.5,0.5,0.5,1.0}; 
 GLfloat white_light_diffuse[]  = {0.5,0.5,0.5,1.0}; 
*/ 
  GLfloat white_light_specular[] = {1.0,1.0,1.0,1.0}; 
 GLfloat white_light_diffuse[]  = {1.0,1.0,1.0,1.0}; 
 

 /* Define modelo de iluminao suave. A intensidade de cor nos pixels  calculada       */ 
 /* com base na interpolao dos atributos definidos nos vrtices durante a rasterizao */ 
 glShadeModel(GL_SMOOTH); 
 /* Habilita materiais coloridos */ 
 glEnable(GL_COLOR_MATERIAL); 
 
 /* Caracteriza o material da superfcie com especular                                   */ 
 glMaterialfv(GL_FRONT, GL_SPECULAR,  mat_specular); 
 /* Define a especularidade do material. Quanto maior a especularidade mais concentrado  */ 
 /*  o highlight                                                                        */ 
 glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess); 
 
 /* Define a posicao da fonte de luz 0 */ 
 glLightfv(GL_LIGHT0, GL_POSITION, light_position1); 
 /* Define a intensidade da componenente difusa   da fonte de luz 1 em cada componente   */ 
 glLightfv(GL_LIGHT0, GL_DIFFUSE, white_light_specular); 
 /* Define a intensidade da componenente specular da fonte de luz 1 em cada componente   */ 
 glLightfv(GL_LIGHT0, GL_SPECULAR, white_light_diffuse); 
 
 /* Define a posicao da fonte de luz 1 */ 
 glLightfv(GL_LIGHT1, GL_POSITION, light_position2); 
 /* Define a intensidade da componenente difusa   da fonte de luz 1 em cada componente   */ 
 glLightfv(GL_LIGHT1, GL_DIFFUSE,  white_light_specular); 
 /* Define a intensidade da componenente specular da fonte de luz 1 em cada componente   */ 
 glLightfv(GL_LIGHT1, GL_SPECULAR, white_light_diffuse); 
  
 /* Habilita iluminao */ 
 glEnable(GL_LIGHTING); 
 /* Habilita fonte de luz 0 */ 
 //glEnable(GL_LIGHT0); 
 /* Habilita fonte de luz 1 */ 
 glEnable(GL_LIGHT1); 
}
 
void DrawTrunk() 
{ 
 glPushMatrix(); 
 glScalef(18,24.0,7.5); 
 glColor3ub(255,245,110); 
 ColorCubeTexturized(texHandle); 
 glPopMatrix(); 
 
} 

void Display(void)
{
   glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 
   glLoadIdentity(); 
   glTranslatef (0.0, 0.0, -150.0); 
   glPushMatrix(); 
   glColor3ub(255,255,255); 
   glTranslatef(0.0,0.0,-300.0); 
   glScalef(200,200,200); 
   glRotatef(-90,1.0,0,0.0); 
   SphereTexturized(earth->textureHandle);; 
   glPopMatrix(); 

   DrawTrunk();
 
   glutSwapBuffers();
}
 
 

void Reshape (int w, int h)
{
   /* Configura o tamanho da janela de desenho como igual o tamanho total   */
   /* do canvas                                                             */
   glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
   /* Seleciona a matriz de modelagem e visualizao*/
   glMatrixMode (GL_PROJECTION);
   /* Carrega a matriz de modelagem e visualizao com a matriz identidade  */
   glLoadIdentity ();
   /* Cria o volume de visualizao(frustum). A especificao do volume de  */
   /* visualizao permite a OpenGL inferir a matriz de projeo            */
   gluPerspective(65.0, (GLfloat) w/(GLfloat)h, 1.0, 500.0); 
   //glOrtho(-75.0,75.0,-75.0,75.0,-500.0,500.0);
   /* Seleciona a matriz de modelagem e visualizao*/
   glMatrixMode(GL_MODELVIEW);
   /* Carrega a matriz de modelagem e visualizao com a matriz identidade  */
   glLoadIdentity(); 
   /* Translada a cmera para a posio (0.0,0.0,150.0). Isto equivale a    */ 
   /* mover os objetos da cena no sentido negativo do eixo z (0.0,0.0,-150) */ 
   //glTranslatef (0.0, 0.0, -150.0);
}
 
// Callback para tratamento de eventos de teclado
void Keyboard (unsigned char key, int x, int y)
{
  
} 
 
void SpecialKeyboard(int key, int x, int y) 
{ 
  
} 
 
void Idle(void) 
{ 

} 

int main(int argc, char** argv)
{  
   /* Inicia a biblioteca Glut */
   glutInit(&argc, argv);
   /* Configura o modo de display para uso de double buffering e 4 */
   /* componentes de cor                                           */
   glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
   /* Configura o tamanho da janela para 500x500 pixels            */
   glutInitWindowSize (500, 500); 
   /* Posiciona a janela na posio (100,100) da tela              */
   glutInitWindowPosition (100, 100);
   /* Cria a janela */
   glutCreateWindow (argv[0]);
   /* Chama uma funo para inicializao geral. Normalmente       */
   /* configura estados e atributos da OpenGL                      */
   Init();
   /* Registra a callback (funo que  chamada para tratamento    */
   /* de eventos da interface) que cuida do desenho do canvas      */
   glutDisplayFunc(Display); 
   /* Registra a callback para que trata do evento associado ao    */
   /* ao redimensionamento da janela                               */
   glutReshapeFunc(Reshape);
   /* Registra a callback para tratamento de eventos gerados pelo  */
   /* teclado                                                      */
   glutKeyboardFunc(Keyboard); 
   glutSpecialFunc(SpecialKeyboard);
   /* Entra no lao que captura e trata eventos da GLUT            */ 
   glutIdleFunc(Idle);
   glutMainLoop();
   return 0;
}
